<?php

namespace Litecms\Powerunit\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litecms\Powerunit\Http\Requests\PowerUnitRequest;
use Litecms\Powerunit\Interfaces\PowerUnitRepositoryInterface;
use Litecms\Powerunit\Models\PowerUnit;

/**
 * Resource controller class for power_unit.
 */
class PowerUnitResourceController extends BaseController
{

    /**
     * Initialize power_unit resource controller.
     *
     * @param type PowerUnitRepositoryInterface $power_unit
     *
     * @return null
     */
    public function __construct(PowerUnitRepositoryInterface $power_unit)
    {
        parent::__construct();
        $this->repository = $power_unit;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litecms\Powerunit\Repositories\Criteria\PowerUnitResourceCriteria::class);
    }

    /**
     * Display a list of power_unit.
     *
     * @return Response
     */
    public function index(PowerUnitRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litecms\Powerunit\Repositories\Presenter\PowerUnitPresenter::class)
                ->$function();
        }

        $power_units = $this->repository->paginate();

        return $this->response->title(trans('powerunit::power_unit.names'))
            ->view('powerunit::power_unit.index', true)
            ->data(compact('power_units', 'view'))
            ->output();
    }

    /**
     * Display power_unit.
     *
     * @param Request $request
     * @param Model   $power_unit
     *
     * @return Response
     */
    public function show(PowerUnitRequest $request, PowerUnit $power_unit)
    {

        if ($power_unit->exists) {
            $view = 'powerunit::power_unit.show';
        } else {
            $view = 'powerunit::power_unit.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('powerunit::power_unit.name'))
            ->data(compact('power_unit'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new power_unit.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(PowerUnitRequest $request)
    {

        $power_unit = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('powerunit::power_unit.name')) 
            ->view('powerunit::power_unit.create', true) 
            ->data(compact('power_unit'))
            ->output();
    }

    /**
     * Create new power_unit.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(PowerUnitRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $power_unit                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('powerunit::power_unit.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('powerunit/power_unit/' . $power_unit->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/powerunit/power_unit'))
                ->redirect();
        }

    }

    /**
     * Show power_unit for editing.
     *
     * @param Request $request
     * @param Model   $power_unit
     *
     * @return Response
     */
    public function edit(PowerUnitRequest $request, PowerUnit $power_unit)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('powerunit::power_unit.name'))
            ->view('powerunit::power_unit.edit', true)
            ->data(compact('power_unit'))
            ->output();
    }

    /**
     * Update the power_unit.
     *
     * @param Request $request
     * @param Model   $power_unit
     *
     * @return Response
     */
    public function update(PowerUnitRequest $request, PowerUnit $power_unit)
    {
        try {
            $attributes = $request->all();

            $power_unit->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('powerunit::power_unit.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('powerunit/power_unit/' . $power_unit->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('powerunit/power_unit/' . $power_unit->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the power_unit.
     *
     * @param Model   $power_unit
     *
     * @return Response
     */
    public function destroy(PowerUnitRequest $request, PowerUnit $power_unit)
    {
        try {

            $power_unit->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('powerunit::power_unit.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('powerunit/power_unit/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('powerunit/power_unit/' . $power_unit->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple power_unit.
     *
     * @param Model   $power_unit
     *
     * @return Response
     */
    public function delete(PowerUnitRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('powerunit::power_unit.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('powerunit/power_unit'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/powerunit/power_unit'))
                ->redirect();
        }

    }

    /**
     * Restore deleted power_units.
     *
     * @param Model   $power_unit
     *
     * @return Response
     */
    public function restore(PowerUnitRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('powerunit::power_unit.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/powerunit/power_unit'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/powerunit/power_unit/'))
                ->redirect();
        }

    }

}
